"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator.throw(value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments)).next());
    });
};
const path = require('path');
require('reflect-metadata');
require('./extend-request');
const metadata_1 = require('../core/metadata');
const util_1 = require('../util');
class RouterReflector {
    constructor(server, router) {
        this.server = server;
        this.router = router;
        this.controllers = {};
    }
    reflectRoutes(controllers) {
        for (var q = 0; q < controllers.length; q++) {
            this.reflectControllerRoutes(controllers[q]);
        }
    }
    reflectControllerRoutes(controllerFn) {
        if (this.controllers[controllerFn])
            throw new Error(`A controller was passed to the router-reflector twice: ${controllerFn}.`);
        var controllerInst = this.controllers[controllerFn] = this.server.injector.resolveInjectable(controllerFn);
        var controllerProto = controllerFn.prototype;
        var meta = Reflect.getOwnMetadata(metadata_1.ControllerMetadataSym, controllerProto);
        if (!meta)
            throw new Error(`Expecting class with @Controller decorator, could not reflect routes for ${controllerProto}.`);
        console.log(`    Reflecting routes for controller ${controllerFn.name}`);
        var routes = Reflect.getOwnMetadata(metadata_1.ControllerRoutesSym, controllerProto) || [];
        for (var q = 0; q < routes.length; q++) {
            var routeFnName = routes[q];
            var routeMeta = Reflect.getOwnMetadata(metadata_1.RouteMetadataSym, controllerProto, routeFnName);
            if (!routeMeta)
                throw new Error(`Could not find route metadata for route ${controllerFn}.${routeFnName}.`);
            this.addRoute(controllerInst, routeFnName, meta, routeMeta);
        }
    }
    addRoute(controller, routeFnName, meta, routeMeta) {
        let policyTypes = [
            ...(this.server.meta.policies || []),
            ...(meta.policies || []),
            ...(routeMeta.policies || [])
        ];
        let policies = policyTypes.map(policyType => this.server.injector.resolveInjectable(policyType));
        if (!util_1.hasNoUndefined(policies))
            throw new Error(`Could not resolve all policies for dependency injection. Controller: ${controller}.${routeFnName}`);
        let boundRoute = controller[routeFnName].bind(controller);
        let fullPath = path.join(...[
            '/',
            this.server.meta.path || '',
            meta.path || '',
            routeMeta.path
        ]);
        console.log(`      Adding route ${routeFnName} (${fullPath})`);
        if (typeof routeMeta.method === 'undefined')
            throw new Error(`Failed to create route ${controller}.${routeFnName}. No method set!`);
        this.router[routeMeta.method](fullPath, this.createFullRouterFn(policyTypes, policies, boundRoute));
    }
    createFullRouterFn(policyTypes, policies, boundRoute) {
        const self = this;
        return function (req, res) {
            return __awaiter(this, void 0, void 0, function* () {
                let allResults = [];
                req.policyResults = self.createPolicyResultsFn(policyTypes, allResults);
                let initialStatusCode = res.statusCode;
                for (var q = 0; q < policies.length; q++) {
                    let policy = policies[q];
                    let result;
                    try {
                        result = yield policy.handle(req, res);
                    }
                    catch (e) {
                        console.error(e);
                        console.log('Serving 500 - Internal server error');
                        res.status(500);
                        res.send('Internal server error');
                        return;
                    }
                    allResults.push(result);
                    if (res.statusCode !== initialStatusCode || res.headersSent)
                        return;
                }
                try {
                    return yield boundRoute(req, res);
                }
                catch (e) {
                    console.error(e);
                    console.log('Serving 500 - Internal server error');
                    res.status(500);
                    res.send('Internal server error');
                }
            });
        };
    }
    createPolicyResultsFn(policyTypes, allResults) {
        return function (policyFn) {
            for (var q = 0; q < policyTypes.length; q++) {
                if (policyTypes[q] === policyFn)
                    return allResults[q];
            }
            return undefined;
        };
    }
}
exports.RouterReflector = RouterReflector;
//# sourceMappingURL=reflector.js.map